@@ -35,3 +35,14 @@ JSON_Parse (
     domLength *byteIndex
     );
 
+domDocument *
+TypedList2DOM (
+    Tcl_Interp *interp,
+    Tcl_Obj *typedList
+    );
+
+int
+isJSONNumber (
+    char *num,
+    domLength numlen
+    );
