@@ -35,8 +35,6 @@
 |   Includes
 |
 \---------------------------------------------------------------------------*/
-#include <tcl.h>
-#include <dom.h>
 #include <tdom.h>
 #include <tcldom.h>
 #include <tclpull.h>
@@ -45,6 +43,13 @@
 
 extern TdomStubs tdomStubs;
 
+#if TCL_MAJOR_VERSION == 8
+# define STUB_VERSION "8.5"
+#else
+# define STUB_VERSION "9.0"
+#endif
+
+
 /*
  *----------------------------------------------------------------------------
  *
@@ -67,7 +72,7 @@ Tdom_Init (
 ) {
         
 #ifdef USE_TCL_STUBS
-    if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
+    if (Tcl_InitStubs(interp, STUB_VERSION, 0) == NULL) {
         return TCL_ERROR;
     }
 #endif
